% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/angle-functions.R
\name{angleToPoint}
\alias{angleToPoint}
\title{Angle with a Reference Point}
\usage{
angleToPoint(x, p = c(1, 1, 1), from = 1, xdiff = diff(x), degrees = TRUE)
}
\arguments{
\item{x}{a single input track; a matrix whose first column is time and whose
remaining columns are a spatial coordinate.}

\item{p}{numeric vector of coordinates of the reference point p to compute angles/distances to.}

\item{from}{index, or vector of indices, of the first row of the track. If
\code{from} is a vector, angles are returned for all steps starting at
the indices in \code{from}.}

\item{xdiff}{row differences of x.}

\item{degrees}{logical; should angles be returned in degrees rather than radians? (default = TRUE).}
}
\value{
A single angle.
}
\description{
Compute the angle between the first step of a track and a reference point. Useful to
detect directed movement towards a point (see examples).
}
\details{
The average angle of steps to a reference point should be 90 degrees if there is
 no bias towards movement in the direction of the reference point. If there is such a bias,
 there should be an enrichment of smaller angles. The expected distribution without bias
 is a uniform distribution in 2D or a sine distribution in 3D (Beltman et al, 2009).
}
\examples{
## Get a distribution of step angles with a reference point
## Use bb to get the corner with highest x,y,and z value
## The histogram is enriched for low angles, suggesting directed movement:
steps <- subtracks( Neutrophils, 1 )
bb <- boundingBox( Neutrophils )
hist( sapply( steps, angleToPoint, p = bb["max",-1] ) )

## The same does not hold for movement of T cells towards the point (0,0,0)
steps <- subtracks( TCells, 1 )
hist( sapply( steps, angleToPoint, p = c(0,0,0) ) )

## Plotting the angle versus the distance to the reference point can also be informative,
## especially when small angles are more frequent at lower distances.
angles <- sapply( steps, angleToPoint, p = bb["max",-1] )
distances <- sapply( steps, distanceToPoint, p = bb["max",-1] )
scatter.smooth( distances, angles )
abline( h = 90, col = "red" )
}
\references{
Joost B. Beltman, Athanasius F.M. Maree and Rob. J. de Boer (2009),
Analysing immune cell migration. \emph{Nature Reviews Immunology} \bold{9},
789--798. doi:10.1038/nri2638
}
\seealso{
\code{\link{distanceToPoint}} to compute the distance to the reference point, and
 \code{\link{AngleAnalysis}} for other methods to compute angles and distances.
}
